/*
 * Copyright (C) 2000 Robert Fitzsimons
 */

#include "datatypes.h"
#include "unicode.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define BUFFER_LENGTH 1024

void init_unicode();
void write_classname(FILE* ofp, kuint8* classfile);
void write_classfile(FILE* ofp, kuint8* classfile);

kuint8* classname;
kuint classname_length;
kuint classname_hash;
kuint8* buffer;
kuint buffer_length;

int main(int argc, char** argv) {
	kuint i;
	FILE* ofp;

	if(argc < 3) {
		return 1;
	}

	init_unicode();

	buffer = malloc(BUFFER_LENGTH);

	if((ofp = fopen(argv[1], "w+")) != NULL) {
		fprintf(ofp, "#include \"datatypes.h\"\n");
		fprintf(ofp, "#include \"classlib.h\"\n\n");

		fprintf(ofp, "struct ClassLib classlib_array[] = {\n");

		for(i = 2; i < argc; i++) {
			write_classname(ofp, argv[i]);
			write_classfile(ofp, argv[i]);
		}

		fprintf(ofp, "\t{0, NULL, 0},\n");
		fprintf(ofp, "};\n\n");

		fclose(ofp);
	} else {
		return 1;
	}

	if(classname != NULL) {
		free(classname);
	}
	if(buffer != NULL) {
		free(buffer);
	}

	return 0;
}

void write_classname(FILE* ofp, kuint8* classfile) {
	kuint length = (kuint)(strchr(classfile, '.')) - (kuint)(classfile);

	if(classname_length < (length + 3)) {
		classname = realloc(classname, (length + 3) * 2);
		(*classname) = 'L';
	}

	memcpy(classname + 1, classfile, length);
	classname_length = length + 2;
	classname[classname_length - 1] = ';';
	classname[classname_length] = '\0';

	classname_hash = unicode_hash_utf8(classname, classname_length, NULL);

	fprintf(ofp, "\t{0x%08X, ", (unsigned int)classname_hash);
}

void write_classfile(FILE* ofp, kuint8* classfile) {
	FILE* ifp;

	fprintf(ofp, "/* %s */ ", classfile);

	if((ifp = fopen(classfile, "r")) != NULL) {
		kuint count;
		kuint i;

		buffer_length = 0;

		fprintf(ofp, "\"");

		do {
			count = fread(buffer, 1, BUFFER_LENGTH, ifp);
			buffer_length += count;

			for(i = 0; i < count; i++) {
				fprintf(ofp, "\\x%02X", buffer[i]);
			}
		} while(count > 0);

		fprintf(ofp, "\", 0x%08X},\n", (unsigned int)buffer_length);

		fclose(ifp);
	} else {
		fprintf(ofp, "NULL, 0},\n");
	}
}

